# Copyright 2016 Jochen Fabricius
# Distribution and modification is encouraged

# evaluate expression for impulse responses
proc :expr {ID op IDorExpr} {
  if {[catch {isID $ID} id1]} {
    return -code error $id1
  }
  set ir1 [getProp $id1 IR]

  if {[catch {isID $IDorExpr} id2]} {
    # is an expression
    if {[catch {set a [expr $IDorExpr]} msg]} {
      return -code error $msg
    }
    esweep::expr ir1 $op $a
  } else {
    # is an ID
    set a [getProp $IDorExpr IR]
    switch -- $op {
      * {
        esweep::convolve ir1 $ir2
      }
      / {
        esweep::deconvolve ir1 $ir2
      }
      {+} -
      {-} {
        esweep::expr ir1 $op $a
      }
      default {
        return -code error "Unknown operator $op"
      }
    }
  }

  ::overlay::modify $id1 [list IR $ir1]

  return -code ok $ID
}

proc :abs {ID} {
  if {[catch {isID $ID} id]} {
    return -code error $id
  }
  set ir [getProp $id IR]
  esweep::fft -inplace -obj ir
  esweep::toPolar -obj ir
  esweep::abs -obj ir
  esweep::expr ir / [esweep::size -obj $ir]
  esweep::ifft -inplace -obj ir
  ::overlay::modify $id [list IR $ir]
  return -code ok $ID
}

if {0} {
proc :clip {mode args} {
  set args [join $args { }]
  if {[llength $args] < 2} {
    return -code error "Need at least two IDs to clip"
  }
  foreach ID $args {
    if {[catch {isID $ID} id]} {
      return -code error $id
    } else {
      lappend ids $id
    }
  }
  set K [getProp [lindex $ids 0] FR]
  switch -- $mode {
    upper {
      foreach id [lrange $ids 1 end] {
        set i 1
        foreach {f1 a1 p1} $K {f2 a2 p2} [getProp $id FR] {
          # amplitude
          lset K $i [expr {$a1 > $a2 ? $a1 : $a2}]
          incr i
          # phase
          lset K $i 0.0
          # skip over frequency
          incr i 2
        }
      }
    }
    lower {
      foreach id [lrange $ids 1 end] {
        set i 1
        foreach {f1 a1 p1} $K {f2 a2 p2} [getProp $id FR] {
          # amplitude
          lset K $i [expr {$a1 < $a2 ? $a1 : $a2}]
          incr i
          # phase
          lset K $i 0.0
          # skip over frequency
          incr i 2
        }
      }
    }
    default {
      return -code error "Unknown clip mode $mode"
    }
  }
  ::overlay::modify [lindex $IDs 0] [list FR $K]
  ::overlay::modify [lindex $IDs 0] [list IR {}]
}
}
