# Copyright 2016 Jochen Fabricius
# Distribution and modification is encouraged

package provide config 1.3

namespace eval ::config  {
  variable options
  array set options {}

  variable flags
  array set flags {}

  variable ensemble {::config}

  proc writeSetup {} {
    variable options
    variable flags
    set dir [file join [pwd] rc]
    if {![file exists $dir]} {
      if {[catch {file mkdir $dir} msg]} {
        return -code error "Can't create resource directory: $msg"
      }
    } elseif {![file isdirectory $dir]} {
        return -code error "Can't create resource files: $dir is not a directory"
    }
    set fp [open [file join [pwd] rc setup] w]
    foreach opt [lsort -dictionary [array names options]] {
      if {$opt eq {} || $options($opt) eq {} || ([info exists flags($opt)] && ([lsearch $flags($opt) {nosafe}] >= 0))} {
        # do not safe this option
        continue
      }

      puts $fp "$opt\t=\t$options($opt)"
    }
    close $fp
  }

  # read/write setup from/to file
  proc readSetup {{strict lazy}} {
    variable options
    variable flags
    if {[file exists [file join [pwd] rc setup]]} {
      set fp [open [file join [pwd] rc setup] r]
      set data [read $fp]
      set data [split $data "\n"]
      set data [join [split $data =]]
      close $fp
      set opt [list]
      foreach tok $data {
        lappend opt [lindex $tok 0] [lrange $tok 1 end]
      }
      if {[llength $opt] <= 0} {
        return 0
      }
      foreach {var val} $opt {
        if {$val eq {}} {continue}
        switch -- $strict {
          strict -
          yes {
            if {[info exists options($var)]} {
              set options($var) $val
            } else {
              return -code error "Option $var not in database"
            }
          }
          no -
          lazy -
          default {
            set options($var) $val
          }
        }
      }
      return 1
    } else {
      return 0
    }
  }

  proc enable_writeOnChange {} {
    ::trace add variable [namespace current]::options write [list apply {{v1 v2 op} {::config::writeSetup}}]
  }

  proc configure {opt valList {flagList {}}} {
    variable options
    variable flags
    if {[info exists options($opt)]} {
      set options($opt) $valList
      foreach f $flagList {
        switch -- $f {
          ro {
            ::config::readonly $opt $valList
          }
          default {}
        }
        lappend flags($opt) $f
      }
      return $valList
    }
    return -code error "Option $opt not in database"
  }

  proc cget {opt} {
    variable options
    if {[info exists options($opt)]} {
      return [join $options($opt)]
    } else {
      return -code error "Option $opt not in database"
    }
  }

  # resemble the array get and set functions
  proc aget {pattern} {
    variable options
    return [array get options $pattern]
  }

  proc aset {list} {
    variable options
    variable flags
    foreach {opt def} $list {
      lassign $def val varFlags
      set options($opt) $val
      foreach f $varFlags {
        switch -- $f {
          ro {
            ::config::readonly $opt $val
          }
          default {}
        }
        lappend flags($opt) $f
      }
    }
  }

  proc ctrace {mod pattern ops cmdPrefix} {
    variable options
    foreach name [array names options $pattern] {
      ::trace $mod variable [namespace current]::options($name) $ops $cmdPrefix
    }
  }

  proc clink {var1 var2} {
    ::trace add variable [namespace current]::options($var1) write [list apply {{link v1 v2 op} {
        upvar $v1 options
        set options($link) $options($v2)
      }} $var2]
    ::trace add variable [namespace current]::options($var2) write [list apply {{link v1 v2 op} {
        upvar $v1 options
        set options($link) $options($v2)
      }} $var1]
  }

  proc readonly {opt val} {
    ::trace add variable [namespace current]::options($opt) write [list apply {{val v1 v2 op} {
        upvar $v1 options
        set options($v2) $val
      }} $val]
  }

  proc makeEnsemble {cmd} {
    namespace ensemble create -command $cmd -subcommands [list configure cget ctrace aget aset append]
  }
}

